@@ -19,39 +19,20 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 #  USA
 
-from __future__ import print_function
+from __future__ import annotations
 
 import fnmatch
 import os
 import warnings
 import weakref
-
-try:
-    from typing import (Any, Callable, Dict, Iterator, List, Optional,
-                        Set, Tuple, Union, cast, KeysView)
-    Any  # pyflakes
-    Callable  # pyflakes
-    Dict  # pyflakes
-    Iterator  # pyflakes
-    KeysView  # pyflakes
-    List  # pyflakes
-    Optional  # pyflakes
-    Set  # pyflakes
-    Tuple  # pyflakes
-    Union  # pyflakes
-except ImportError:
-    def cast(typ, obj):  # type: ignore
-        return obj
-    pass
+from collections.abc import Callable, Iterator, KeysView
+from typing import Any, cast
 
 import apt_pkg
-from apt.package import Package, Version
+
 import apt.progress.text
+from apt.package import Package, Version
 from apt.progress.base import AcquireProgress, InstallProgress, OpProgress
-OpProgress  # pyflakes
-InstallProgress  # pyflakes
-AcquireProgress  # pyflakes
-Version  # pyflakes
 
 
 class FetchCancelledException(IOError):
@@ -74,30 +55,28 @@ class CacheClosedException(Exception):
     """Exception that is thrown when the cache is used after close()."""
 
 
-class _WrappedLock(object):
+class _WrappedLock:
     """Wraps an apt_pkg.FileLock to raise LockFailedException.
 
     Initialized using a directory path."""
 
-    def __init__(self, path):
-        # type: (str) -> None
+    def __init__(self, path: str) -> None:
         self._path = path
         self._lock = apt_pkg.FileLock(os.path.join(path, "lock"))
 
-    def __enter__(self):
-        # type: () -> None
+    def __enter__(self) -> None:
         try:
             return self._lock.__enter__()
         except apt_pkg.Error as e:
-            raise LockFailedException(("Failed to lock directory %s: %s") %
-                                       (self._path, e))
+            raise LockFailedException(
+                ("Failed to lock directory %s: %s") % (self._path, e)
+            )
 
-    def __exit__(self, typ, value, traceback):
-        # type: (object, object, object) -> None
+    def __exit__(self, typ: object, value: object, traceback: object) -> None:
         return self._lock.__exit__(typ, value, traceback)
 
 
-class Cache(object):
+class Cache:
     """Dictionary-like package cache.
 
     The APT cache file contains a hash table mapping names of binary
@@ -123,18 +102,28 @@ class Cache(object):
         in :meth:`keys()`, though, to keep :meth:`keys()` a unique set.
     """
 
-    def __init__(self, progress=None, rootdir=None, memonly=False):
-        # type: (OpProgress, str, bool) -> None
-        self._cache = cast(apt_pkg.Cache, None)  # type: apt_pkg.Cache
-        self._depcache = cast(apt_pkg.DepCache, None)  # type: apt_pkg.DepCache
-        self._records = cast(apt_pkg.PackageRecords, None)  # type: apt_pkg.PackageRecords # nopep8
-        self._list = cast(apt_pkg.SourceList, None)  # type: apt_pkg.SourceList
-        self._callbacks = {}  # type: Dict[str, List[Union[Callable[..., None],str]]] # nopep8
-        self._callbacks2 = {}  # type: Dict[str, List[Tuple[Callable[..., Any], Tuple[Any, ...], Dict[Any,Any]]]] # nopep8
-        self._weakref = weakref.WeakValueDictionary()  # type: weakref.WeakValueDictionary[str, apt.Package] # nopep8
-        self._weakversions = weakref.WeakSet()  # type: weakref.WeakSet[Version] # nopep8
+    def __init__(
+        self,
+        progress: OpProgress | None = None,
+        rootdir: str | None = None,
+        memonly: bool = False,
+    ) -> None:
+        self._cache: apt_pkg.Cache = cast(apt_pkg.Cache, None)
+        self._depcache: apt_pkg.DepCache = cast(apt_pkg.DepCache, None)
+        self._records: apt_pkg.PackageRecords = cast(
+            apt_pkg.PackageRecords, None
+        )  # noqa
+        self._list: apt_pkg.SourceList = cast(apt_pkg.SourceList, None)
+        self._callbacks: dict[str, list[Callable[..., None] | str]] = {}  # noqa
+        self._callbacks2: dict[
+            str, list[tuple[Callable[..., Any], tuple[Any, ...], dict[Any, Any]]]
+        ] = {}  # noqa
+        self._weakref: weakref.WeakValueDictionary[
+            str, apt.Package
+        ] = weakref.WeakValueDictionary()  # noqa
+        self._weakversions: weakref.WeakSet[Version] = weakref.WeakSet()  # noqa
         self._changes_count = -1
-        self._sorted_set = None  # type: Optional[List[str]]
+        self._sorted_set: list[str] | None = None
 
         self.connect("cache_post_open", "_inc_changes_count")
         self.connect("cache_post_change", "_inc_changes_count")
@@ -144,18 +133,16 @@ class Cache(object):
         if rootdir:
             rootdir = os.path.abspath(rootdir)
             if os.path.exists(rootdir + "/etc/apt/apt.conf"):
-                apt_pkg.read_config_file(apt_pkg.config,
-                                         rootdir + "/etc/apt/apt.conf")
+                apt_pkg.read_config_file(apt_pkg.config, rootdir + "/etc/apt/apt.conf")
             if os.path.isdir(rootdir + "/etc/apt/apt.conf.d"):
-                apt_pkg.read_config_dir(apt_pkg.config,
-                                        rootdir + "/etc/apt/apt.conf.d")
+                apt_pkg.read_config_dir(apt_pkg.config, rootdir + "/etc/apt/apt.conf.d")
             apt_pkg.config.set("Dir", rootdir)
-            apt_pkg.config.set("Dir::State::status",
-                               rootdir + "/var/lib/dpkg/status")
+            apt_pkg.config.set("Dir::State::status", rootdir + "/var/lib/dpkg/status")
             # also set dpkg to the rootdir path so that its called for the
             # --print-foreign-architectures call
-            apt_pkg.config.set("Dir::bin::dpkg",
-                               os.path.join(rootdir, "usr", "bin", "dpkg"))
+            apt_pkg.config.set(
+                "Dir::bin::dpkg", os.path.join(rootdir, "usr", "bin", "dpkg")
+            )
             # create required dirs/files when run with special rootdir
             # automatically
             self._check_and_create_required_dirs(rootdir)
@@ -169,56 +156,53 @@ class Cache(object):
 
         self.open(progress)
 
-    def fix_broken(self):
-        # type: () -> None
+    def fix_broken(self) -> None:
         """Fix broken packages."""
         self._depcache.fix_broken()
 
-    def _inc_changes_count(self):
-        # type: () -> None
+    def _inc_changes_count(self) -> None:
         """Increase the number of changes"""
         self._changes_count += 1
 
-    def _check_and_create_required_dirs(self, rootdir):
-        # type: (str) -> None
+    def _check_and_create_required_dirs(self, rootdir: str) -> None:
         """
         check if the required apt directories/files are there and if
         not create them
         """
-        files = ["/var/lib/dpkg/status",
-                 "/etc/apt/sources.list",
-                 ]
-        dirs = ["/var/lib/dpkg",
-                "/etc/apt/",
-                "/var/cache/apt/archives/partial",
-                "/var/lib/apt/lists/partial",
-                ]
+        files = [
+            "/var/lib/dpkg/status",
+            "/etc/apt/sources.list",
+        ]
+        dirs = [
+            "/var/lib/dpkg",
+            "/etc/apt/",
+            "/var/cache/apt/archives/partial",
+            "/var/lib/apt/lists/partial",
+        ]
         for d in dirs:
             if not os.path.exists(rootdir + d):
-                #print "creating: ", rootdir + d
+                # print "creating: ", rootdir + d
                 os.makedirs(rootdir + d)
         for f in files:
             if not os.path.exists(rootdir + f):
                 open(rootdir + f, "w").close()
 
-    def _run_callbacks(self, name):
-        # type: (str) -> None
-        """ internal helper to run a callback """
+    def _run_callbacks(self, name: str) -> None:
+        """internal helper to run a callback"""
         if name in self._callbacks:
             for callback in self._callbacks[name]:
-                if callback == '_inc_changes_count':
+                if callback == "_inc_changes_count":
                     self._inc_changes_count()
                 else:
                     callback()  # type: ignore
 
         if name in self._callbacks2:
             for callback, args, kwds in self._callbacks2[name]:
-                    callback(self, *args, **kwds)
+                callback(self, *args, **kwds)
 
-    def open(self, progress=None):
-        # type: (OpProgress) -> None
-        """ Open the package cache, after that it can be used like
-            a dictionary
+    def open(self, progress: OpProgress | None = None) -> None:
+        """Open the package cache, after that it can be used like
+        a dictionary
         """
         if progress is None:
             progress = apt.progress.base.OpProgress()
@@ -240,8 +224,7 @@ class Cache(object):
         progress.done()
         self._run_callbacks("cache_post_open")
 
-    def __remap(self):
-        # type: () -> None
+    def __remap(self) -> None:
         """Called after cache reopen() to relocate to new cache.
 
         Relocate objects like packages and versions from the old
@@ -262,52 +245,48 @@ class Cache(object):
             # Package has been reseated above, reseat version
             for v in ver.package._pkg.version_list:
                 # Requirements as in debListParser::SameVersion
-                if (v.hash == ver._cand.hash and
-                    (v.size == 0 or ver._cand.size == 0 or
-                     v.size == ver._cand.size) and
-                    v.multi_arch == ver._cand.multi_arch and
-                    v.ver_str == ver._cand.ver_str):
+                if (
+                    v.hash == ver._cand.hash
+                    and (v.size == 0 or ver._cand.size == 0 or v.size == ver._cand.size)
+                    and v.multi_arch == ver._cand.multi_arch
+                    and v.ver_str == ver._cand.ver_str
+                ):
                     ver._cand = v
                     break
             else:
                 self._weakversions.remove(ver)
 
-    def close(self):
-        # type: () -> None
-        """ Close the package cache """
+    def close(self) -> None:
+        """Close the package cache"""
         # explicitely free the FDs that _records has open
         del self._records
         self._records = cast(apt_pkg.PackageRecords, None)
 
-    def __enter__(self):
-        # type: () -> Cache
-        """ Enter the with statement """
+    def __enter__(self) -> Cache:
+        """Enter the with statement"""
         return self
 
-    def __exit__(self, exc_type, exc_value, traceback):
-        # type: (object, object, object) -> None
-        """ Exit the with statement """
+    def __exit__(self, exc_type: object, exc_value: object, traceback: object) -> None:
+        """Exit the with statement"""
         self.close()
 
-    def __getitem__(self, key):
-        # type: (object) -> Package
-        """ look like a dictionary (get key) """
+    def __getitem__(self, key: object) -> Package:
+        """look like a dictionary (get key)"""
         try:
             key = str(key)
             rawpkg = self._cache[key]
         except KeyError:
-            raise KeyError('The cache has no package named %r' % key)
+            raise KeyError("The cache has no package named %r" % key)
 
         # It might be excluded due to not having a version or something
         if not self.__is_real_pkg(rawpkg):
-            raise KeyError('The cache has no package named %r' % key)
+            raise KeyError("The cache has no package named %r" % key)
 
         pkg = self._rawpkg_to_pkg(rawpkg)
 
         return pkg
 
-    def get(self, key, default=None):
-        # type: (object, object) -> Any
+    def get(self, key: object, default: object = None) -> Any:
         """Return *self*[*key*] or *default* if *key* not in *self*.
 
         .. versionadded:: 1.1
@@ -317,8 +296,7 @@ class Cache(object):
         except KeyError:
             return default
 
-    def _rawpkg_to_pkg(self, rawpkg):
-        # type: (apt_pkg.Package) -> Package
+    def _rawpkg_to_pkg(self, rawpkg: apt_pkg.Package) -> Package:
         """Returns the apt.Package object for an apt_pkg.Package object.
 
         .. versionadded:: 1.0.0
@@ -327,8 +305,7 @@ class Cache(object):
 
         return self._weakref.setdefault(fullname, Package(self, rawpkg))
 
-    def __iter__(self):
-        # type: () -> Iterator[Package]
+    def __iter__(self) -> Iterator[Package]:
         # We iterate sorted over package names here. With this we read the
         # package lists linearly if we need to access the package records,
         # instead of having to do thousands of random seeks; the latter
@@ -338,37 +315,33 @@ class Cache(object):
             pkg = Package(self, self._cache[pkgname])
             yield self._weakref.setdefault(pkgname, pkg)
 
-    def __is_real_pkg(self, rawpkg):
-        # type: (apt_pkg.Package) -> bool
+    def __is_real_pkg(self, rawpkg: apt_pkg.Package) -> bool:
         """Check if the apt_pkg.Package provided is a real package."""
         return rawpkg.has_versions
 
-    def has_key(self, key):
-        # type: (object) -> bool
+    def has_key(self, key: object) -> bool:
         return key in self
 
-    def __contains__(self, key):
-        # type: (object) -> bool
+    def __contains__(self, key: object) -> bool:
         try:
             return self.__is_real_pkg(self._cache[str(key)])
         except KeyError:
             return False
 
-    def __len__(self):
-        # type: () -> int
+    def __len__(self) -> int:
         return len(self.keys())
 
-    def keys(self):
-        # type: () -> List[str]
+    def keys(self) -> list[str]:
         if self._sorted_set is None:
-            self._sorted_set = sorted(p.get_fullname(pretty=True)
-                                      for p in self._cache.packages
-                                        if self.__is_real_pkg(p))
+            self._sorted_set = sorted(
+                p.get_fullname(pretty=True)
+                for p in self._cache.packages
+                if self.__is_real_pkg(p)
+            )
         return list(self._sorted_set)  # We need a copy here, caller may modify
 
-    def get_changes(self):
-        # type: () -> List[Package]
-        """ Get the marked changes """
+    def get_changes(self) -> list[Package]:
+        """Get the marked changes"""
         changes = []
         marked_keep = self._depcache.marked_keep
         for rawpkg in self._cache.packages:
@@ -376,8 +349,7 @@ class Cache(object):
                 changes.append(self._rawpkg_to_pkg(rawpkg))
         return changes
 
-    def upgrade(self, dist_upgrade=False):
-        # type: (bool) -> None
+    def upgrade(self, dist_upgrade: bool = False) -> None:
         """Upgrade all packages.
 
         If the parameter *dist_upgrade* is True, new dependencies will be
@@ -389,47 +361,47 @@ class Cache(object):
         self.cache_post_change()
 
     @property
-    def required_download(self):
-        # type: () -> int
+    def required_download(self) -> int:
         """Get the size of the packages that are required to download."""
         if self._records is None:
-            raise CacheClosedException(
-                "Cache object used after close() called")
+            raise CacheClosedException("Cache object used after close() called")
         pm = apt_pkg.PackageManager(self._depcache)
         fetcher = apt_pkg.Acquire()
         pm.get_archives(fetcher, self._list, self._records)
         return fetcher.fetch_needed
 
     @property
-    def required_space(self):
-        # type: () -> int
+    def required_space(self) -> int:
         """Get the size of the additional required space on the fs."""
         return self._depcache.usr_size
 
     @property
-    def req_reinstall_pkgs(self):
-        # type: () -> Set[str]
+    def req_reinstall_pkgs(self) -> set[str]:
         """Return the packages not downloadable packages in reqreinst state."""
         reqreinst = set()
         get_candidate_ver = self._depcache.get_candidate_ver
-        states = frozenset((apt_pkg.INSTSTATE_REINSTREQ,
-                            apt_pkg.INSTSTATE_HOLD_REINSTREQ))
+        states = frozenset(
+            (apt_pkg.INSTSTATE_REINSTREQ, apt_pkg.INSTSTATE_HOLD_REINSTREQ)
+        )
         for pkg in self._cache.packages:
             cand = get_candidate_ver(pkg)
             if cand and not cand.downloadable and pkg.inst_state in states:
                 reqreinst.add(pkg.get_fullname(pretty=True))
         return reqreinst
 
-    def _run_fetcher(self, fetcher, allow_unauthenticated):
-        # type: (apt_pkg.Acquire, Optional[bool]) -> int
+    def _run_fetcher(
+        self, fetcher: apt_pkg.Acquire, allow_unauthenticated: bool | None
+    ) -> int:
         if allow_unauthenticated is None:
-            allow_unauthenticated = apt_pkg.config.find_b("APT::Get::"
-                                        "AllowUnauthenticated", False)
+            allow_unauthenticated = apt_pkg.config.find_b(
+                "APT::Get::" "AllowUnauthenticated", False
+            )
 
         untrusted = [item for item in fetcher.items if not item.is_trusted]
         if untrusted and not allow_unauthenticated:
-            raise UntrustedException("Untrusted packages:\n%s" %
-                "\n".join(i.desc_uri for i in untrusted))
+            raise UntrustedException(
+                "Untrusted packages:\n%s" % "\n".join(i.desc_uri for i in untrusted)
+            )
 
         # do the actual fetching
         res = fetcher.run()
@@ -442,8 +414,7 @@ class Cache(object):
                 continue
             if item.STAT_IDLE:
                 continue
-            err_msg += "Failed to fetch %s %s\n" % (item.desc_uri,
-                                                    item.error_text)
+            err_msg += f"Failed to fetch {item.desc_uri} {item.error_text}\n"
             failed = True
 
         # we raise a exception if the download failed or it was cancelt
@@ -453,16 +424,15 @@ class Cache(object):
             raise FetchFailedException(err_msg)
         return res
 
-    def _fetch_archives(self,
-                        fetcher,                # type: apt_pkg.Acquire
-                        pm,                     # type: apt_pkg.PackageManager
-                        allow_unauthenticated=None,  # type: Optional[bool]
-                        ):
-        # type: (...) -> int
-        """ fetch the needed archives """
+    def _fetch_archives(
+        self,
+        fetcher: apt_pkg.Acquire,
+        pm: apt_pkg.PackageManager,
+        allow_unauthenticated: bool | None = None,
+    ) -> int:
+        """fetch the needed archives"""
         if self._records is None:
-            raise CacheClosedException(
-                "Cache object used after close() called")
+            raise CacheClosedException("Cache object used after close() called")
 
         # this may as well throw a SystemError exception
         if not pm.get_archives(fetcher, self._list, self._records):
@@ -472,12 +442,12 @@ class Cache(object):
         # fetched
         return self._run_fetcher(fetcher, allow_unauthenticated)
 
-    def fetch_archives(self,
-                       progress=None,        # type: Optional[AcquireProgress]
-                       fetcher=None,         # type: Optional[apt_pkg.Acquire]
-                       allow_unauthenticated=None,  # type: Optional[bool]
-                       ):
-        # type: (...) -> int
+    def fetch_archives(
+        self,
+        progress: AcquireProgress | None = None,
+        fetcher: apt_pkg.Acquire | None = None,
+        allow_unauthenticated: bool | None = None,
+    ) -> int:
         """Fetch the archives for all packages marked for install/upgrade.
 
         You can specify either an :class:`apt.progress.base.AcquireProgress()`
@@ -502,12 +472,11 @@ class Cache(object):
             fetcher = apt_pkg.Acquire(progress)
 
         with self._archive_lock:
-            return self._fetch_archives(fetcher,
-                                        apt_pkg.PackageManager(self._depcache),
-                                        allow_unauthenticated)
+            return self._fetch_archives(
+                fetcher, apt_pkg.PackageManager(self._depcache), allow_unauthenticated
+            )
 
-    def is_virtual_package(self, pkgname):
-        # type: (str) -> bool
+    def is_virtual_package(self, pkgname: str) -> bool:
         """Return whether the package is a virtual package."""
         try:
             pkg = self._cache[pkgname]
@@ -516,9 +485,12 @@ class Cache(object):
         else:
             return bool(pkg.has_provides and not pkg.has_versions)
 
-    def get_providing_packages(self, pkgname, candidate_only=True,
-                               include_nonvirtual=False):
-        # type: (str, bool, bool) -> List[Package]
+    def get_providing_packages(
+        self,
+        pkgname: str,
+        candidate_only: bool = True,
+        include_nonvirtual: bool = False,
+    ) -> list[Package]:
         """Return a list of all packages providing a package.
 
         Return a list of packages which provide the virtual package of the
@@ -534,7 +506,7 @@ class Cache(object):
         a virtual pkg.
         """
 
-        providers = set()  # type: Set[Package]
+        providers: set[Package] = set()
         get_candidate_ver = self._depcache.get_candidate_ver
         try:
             vp = self._cache[pkgname]
@@ -549,9 +521,13 @@ class Cache(object):
                 providers.add(self._rawpkg_to_pkg(rawpkg))
         return list(providers)
 
-    def update(self, fetch_progress=None, pulse_interval=0,
-               raise_on_error=True, sources_list=None):
-        # type: (AcquireProgress, int, bool, str) -> int
+    def update(
+        self,
+        fetch_progress: AcquireProgress | None = None,
+        pulse_interval: int = 0,
+        raise_on_error: bool = True,
+        sources_list: str | None = None,
+    ) -> int:
         """Run the equivalent of apt-get update.
 
         You probably want to call open() afterwards, in order to utilise the
@@ -567,11 +543,11 @@ class Cache(object):
         with _WrappedLock(apt_pkg.config.find_dir("Dir::State::Lists")):
             if sources_list:
                 old_sources_list = apt_pkg.config.find("Dir::Etc::sourcelist")
-                old_sources_list_d = (
-                    apt_pkg.config.find("Dir::Etc::sourceparts"))
+                old_sources_list_d = apt_pkg.config.find("Dir::Etc::sourceparts")
                 old_cleanup = apt_pkg.config.find("APT::List-Cleanup")
-                apt_pkg.config.set("Dir::Etc::sourcelist",
-                                   os.path.abspath(sources_list))
+                apt_pkg.config.set(
+                    "Dir::Etc::sourcelist", os.path.abspath(sources_list)
+                )
                 apt_pkg.config.set("Dir::Etc::sourceparts", "xxx")
                 apt_pkg.config.set("APT::List-Cleanup", "0")
                 slist = apt_pkg.SourceList()
@@ -583,8 +559,7 @@ class Cache(object):
                 if fetch_progress is None:
                     fetch_progress = apt.progress.base.AcquireProgress()
                 try:
-                    res = self._cache.update(fetch_progress, slist,
-                                             pulse_interval)
+                    res = self._cache.update(fetch_progress, slist, pulse_interval)
                 except SystemError as e:
                     raise FetchFailedException(e)
                 if not res and raise_on_error:
@@ -593,15 +568,13 @@ class Cache(object):
                     return res
             finally:
                 if sources_list:
-                    apt_pkg.config.set("Dir::Etc::sourcelist",
-                                       old_sources_list)
-                    apt_pkg.config.set("Dir::Etc::sourceparts",
-                                       old_sources_list_d)
-                    apt_pkg.config.set("APT::List-Cleanup",
-                                       old_cleanup)
+                    apt_pkg.config.set("Dir::Etc::sourcelist", old_sources_list)
+                    apt_pkg.config.set("Dir::Etc::sourceparts", old_sources_list_d)
+                    apt_pkg.config.set("APT::List-Cleanup", old_cleanup)
 
-    def install_archives(self, pm, install_progress):
-        # type: (apt_pkg.PackageManager, InstallProgress) -> int
+    def install_archives(
+        self, pm: apt_pkg.PackageManager, install_progress: InstallProgress
+    ) -> int:
         """
         The first parameter *pm* refers to an object returned by
         apt_pkg.PackageManager().
@@ -634,12 +607,12 @@ class Cache(object):
             install_progress.finish_update()
         return res
 
-    def commit(self,
-               fetch_progress=None,         # type: Optional[AcquireProgress]
-               install_progress=None,       # type: Optional[InstallProgress]
-               allow_unauthenticated=None,  # type: Optional[bool]
-               ):
-        # type: (...) -> bool
+    def commit(
+        self,
+        fetch_progress: AcquireProgress | None = None,
+        install_progress: InstallProgress | None = None,
+        allow_unauthenticated: bool | None = None,
+    ) -> bool:
         """Apply the marked changes to the cache.
 
         The first parameter, *fetch_progress*, refers to a FetchProgress()
@@ -673,8 +646,7 @@ class Cache(object):
             with self._archive_lock:
                 while True:
                     # fetch archives first
-                    res = self._fetch_archives(fetcher, pm,
-                                               allow_unauthenticated)
+                    res = self._fetch_archives(fetcher, pm, allow_unauthenticated)
 
                     # then install
                     res = self.install_archives(pm, install_progress)
@@ -685,33 +657,30 @@ class Cache(object):
                     elif res == pm.RESULT_INCOMPLETE:
                         pass
                     else:
-                        raise SystemError("internal-error: unknown result "
-                                          "code from InstallArchives: %s" %
-                                          res)
+                        raise SystemError(
+                            "internal-error: unknown result "
+                            "code from InstallArchives: %s" % res
+                        )
                     # reload the fetcher for media swaping
                     fetcher.shutdown()
-        return (res == pm.RESULT_COMPLETED)
+        return res == pm.RESULT_COMPLETED
 
-    def clear(self):
-        # type: () -> None
-        """ Unmark all changes """
+    def clear(self) -> None:
+        """Unmark all changes"""
         self._depcache.init()
 
     # cache changes
 
-    def cache_post_change(self):
-        # type: () -> None
-        " called internally if the cache has changed, emit a signal then "
+    def cache_post_change(self) -> None:
+        "called internally if the cache has changed, emit a signal then"
         self._run_callbacks("cache_post_change")
 
-    def cache_pre_change(self):
-        # type: () -> None
-        """ called internally if the cache is about to change, emit
-            a signal then """
+    def cache_pre_change(self) -> None:
+        """called internally if the cache is about to change, emit
+        a signal then"""
         self._run_callbacks("cache_pre_change")
 
-    def connect(self, name, callback):
-        # type: (str, Union[Callable[..., None],str]) -> None
+    def connect(self, name: str, callback: Callable[..., None] | str) -> None:
         """Connect to a signal.
 
         .. deprecated:: 1.0
@@ -719,15 +688,19 @@ class Cache(object):
             Please use connect2() instead, as this function is very
             likely to cause a memory leak.
         """
-        if callback != '_inc_changes_count':
-            warnings.warn("connect() likely causes a reference"
-                          " cycle, use connect2() instead", RuntimeWarning, 2)
+        if callback != "_inc_changes_count":
+            warnings.warn(
+                "connect() likely causes a reference" " cycle, use connect2() instead",
+                RuntimeWarning,
+                2,
+            )
         if name not in self._callbacks:
             self._callbacks[name] = []
         self._callbacks[name].append(callback)
 
-    def connect2(self, name, callback, *args, **kwds):
-        # type: (str, Callable[..., Any], object, object) -> None
+    def connect2(
+        self, name: str, callback: Callable[..., Any], *args: object, **kwds: object
+    ) -> None:
         """Connect to a signal.
 
         The callback will be passed the cache as an argument, and
@@ -747,8 +720,7 @@ class Cache(object):
             self._callbacks2[name] = []
         self._callbacks2[name].append((callback, args, kwds))
 
-    def actiongroup(self):
-        # type: () -> apt_pkg.ActionGroup
+    def actiongroup(self) -> apt_pkg.ActionGroup:
         """Return an `ActionGroup` object for the current cache.
 
         Action groups can be used to speedup actions. The action group is
@@ -770,8 +742,7 @@ class Cache(object):
         return apt_pkg.ActionGroup(self._depcache)
 
     @property
-    def dpkg_journal_dirty(self):
-        # type: () -> bool
+    def dpkg_journal_dirty(self) -> bool:
         """Return True if the dpkg was interrupted
 
         All dpkg operations will fail until this is fixed, the action to
@@ -779,77 +750,63 @@ class Cache(object):
         'dpkg --configure -a' as root.
         """
         dpkg_status_dir = os.path.dirname(
-            apt_pkg.config.find_file("Dir::State::status"))
+            apt_pkg.config.find_file("Dir::State::status")
+        )
         for f in os.listdir(os.path.join(dpkg_status_dir, "updates")):
             if fnmatch.fnmatch(f, "[0-9]*"):
                 return True
         return False
 
     @property
-    def broken_count(self):
-        # type: () -> int
+    def broken_count(self) -> int:
         """Return the number of packages with broken dependencies."""
         return self._depcache.broken_count
 
     @property
-    def delete_count(self):
-        # type: () -> int
+    def delete_count(self) -> int:
         """Return the number of packages marked for deletion."""
         return self._depcache.del_count
 
     @property
-    def install_count(self):
-        # type: () -> int
+    def install_count(self) -> int:
         """Return the number of packages marked for installation."""
         return self._depcache.inst_count
 
     @property
-    def keep_count(self):
-        # type: () -> int
+    def keep_count(self) -> int:
         """Return the number of packages marked as keep."""
         return self._depcache.keep_count
 
 
-class ProblemResolver(object):
+class ProblemResolver:
     """Resolve problems due to dependencies and conflicts.
 
     The first argument 'cache' is an instance of apt.Cache.
     """
 
-    def __init__(self, cache):
-        # type: (Cache) -> None
+    def __init__(self, cache: Cache) -> None:
         self._resolver = apt_pkg.ProblemResolver(cache._depcache)
         self._cache = cache
 
-    def clear(self, package):
-        # type: (Package) -> None
+    def clear(self, package: Package) -> None:
         """Reset the package to the default state."""
         self._resolver.clear(package._pkg)
 
-    def install_protect(self):
-        # type: () -> None
-        """mark protected packages for install or removal."""
-        self._resolver.install_protect()
-
-    def protect(self, package):
-        # type: (Package) -> None
+    def protect(self, package: Package) -> None:
         """Protect a package so it won't be removed."""
         self._resolver.protect(package._pkg)
 
-    def remove(self, package):
-        # type: (Package) -> None
+    def remove(self, package: Package) -> None:
         """Mark a package for removal."""
         self._resolver.remove(package._pkg)
 
-    def resolve(self):
-        # type: () -> None
+    def resolve(self) -> None:
         """Resolve dependencies, try to remove packages where needed."""
         self._cache.cache_pre_change()
         self._resolver.resolve()
         self._cache.cache_post_change()
 
-    def resolve_by_keep(self):
-        # type: () -> None
+    def resolve_by_keep(self) -> None:
         """Resolve dependencies, do not try to remove packages."""
         self._cache.cache_pre_change()
         self._resolver.resolve_by_keep()
@@ -859,22 +816,20 @@ class ProblemResolver(object):
 # ----------------------------- experimental interface
 
 
-class Filter(object):
-    """ Filter base class """
+class Filter:
+    """Filter base class"""
 
-    def apply(self, pkg):
-        # type: (Package) -> bool
-        """ Filter function, return True if the package matchs a
-            filter criteria and False otherwise
+    def apply(self, pkg: Package) -> bool:
+        """Filter function, return True if the package matchs a
+        filter criteria and False otherwise
         """
         return True
 
 
 class MarkedChangesFilter(Filter):
-    """ Filter that returns all marked changes """
+    """Filter that returns all marked changes"""
 
-    def apply(self, pkg):
-        # type: (Package) -> bool
+    def apply(self, pkg: Package) -> bool:
         if pkg.marked_install or pkg.marked_delete or pkg.marked_upgrade:
             return True
         else:
@@ -887,26 +842,23 @@ class InstalledFilter(Filter):
     .. versionadded:: 1.0.0
     """
 
-    def apply(self, pkg):
-        # type: (Package) -> bool
+    def apply(self, pkg: Package) -> bool:
         return pkg.is_installed
 
 
-class _FilteredCacheHelper(object):
+class _FilteredCacheHelper:
     """Helper class for FilteredCache to break a reference cycle."""
 
-    def __init__(self, cache):
-        # type: (Cache) -> None
+    def __init__(self, cache: Cache) -> None:
         # Do not keep a reference to the cache, or you have a cycle!
 
-        self._filtered = {}  # type: Dict[str,bool]
-        self._filters = []  # type: List[Filter]
+        self._filtered: dict[str, bool] = {}
+        self._filters: list[Filter] = []
         cache.connect2("cache_post_change", self.filter_cache_post_change)
         cache.connect2("cache_post_open", self.filter_cache_post_change)
 
-    def _reapply_filter(self, cache):
-        # type: (Cache) -> None
-        " internal helper to refilter "
+    def _reapply_filter(self, cache: Cache) -> None:
+        "internal helper to refilter"
         # Do not keep a reference to the cache, or you have a cycle!
         self._filtered = {}
         for pkg in cache:
@@ -915,98 +867,85 @@ class _FilteredCacheHelper(object):
                     self._filtered[pkg.name] = True
                     break
 
-    def set_filter(self, filter):
-        # type: (Filter) -> None
+    def set_filter(self, filter: Filter) -> None:
         """Set the current active filter."""
         self._filters = []
         self._filters.append(filter)
 
-    def filter_cache_post_change(self, cache):
-        # type: (Cache) -> None
+    def filter_cache_post_change(self, cache: Cache) -> None:
         """Called internally if the cache changes, emit a signal then."""
         # Do not keep a reference to the cache, or you have a cycle!
         self._reapply_filter(cache)
 
 
-class FilteredCache(object):
-    """ A package cache that is filtered.
+class FilteredCache:
+    """A package cache that is filtered.
 
-        Can work on a existing cache or create a new one
+    Can work on a existing cache or create a new one
     """
 
-    def __init__(self, cache=None, progress=None):
-        # type: (Cache, OpProgress) -> None
+    def __init__(
+        self, cache: Cache | None = None, progress: OpProgress | None = None
+    ) -> None:
         if cache is None:
             self.cache = Cache(progress)
         else:
             self.cache = cache
         self._helper = _FilteredCacheHelper(self.cache)
 
-    def __len__(self):
-        # type: () -> int
+    def __len__(self) -> int:
         return len(self._helper._filtered)
 
-    def __getitem__(self, key):
-        # type: (str) -> Package
+    def __getitem__(self, key: str) -> Package:
         return self.cache[key]
 
-    def __iter__(self):
-        # type: () -> Iterator[Package]
+    def __iter__(self) -> Iterator[Package]:
         for pkgname in self._helper._filtered:
             yield self.cache[pkgname]
 
-    def keys(self):
-        # type: () -> KeysView[str]
+    def keys(self) -> KeysView[str]:
         return self._helper._filtered.keys()
 
-    def has_key(self, key):
-        # type: (object) -> bool
+    def has_key(self, key: object) -> bool:
         return key in self
 
-    def __contains__(self, key):
-        # type: (object) -> bool
+    def __contains__(self, key: object) -> bool:
         try:
             # Normalize package name for multi arch
             return self.cache[key].name in self._helper._filtered
         except KeyError:
             return False
 
-    def set_filter(self, filter):
-        # type: (Filter) -> None
+    def set_filter(self, filter: Filter) -> None:
         """Set the current active filter."""
         self._helper.set_filter(filter)
         self.cache.cache_post_change()
 
-    def filter_cache_post_change(self):
-        # type: () -> None
+    def filter_cache_post_change(self) -> None:
         """Called internally if the cache changes, emit a signal then."""
         self._helper.filter_cache_post_change(self.cache)
 
-    def __getattr__(self, key):
-        # type: (str) -> Any
+    def __getattr__(self, key: str) -> Any:
         """we try to look exactly like a real cache."""
         return getattr(self.cache, key)
 
 
-def cache_pre_changed(cache):
-    # type: (Cache) -> None
+def cache_pre_changed(cache: Cache) -> None:
     print("cache pre changed")
 
 
-def cache_post_changed(cache):
-    # type: (Cache) -> None
+def cache_post_changed(cache: Cache) -> None:
     print("cache post changed")
 
 
-def _test():
-    # type: () -> None
+def _test() -> None:
     """Internal test code."""
     print("Cache self test")
     apt_pkg.init()
     cache = Cache(apt.progress.text.OpProgress())
     cache.connect2("cache_pre_change", cache_pre_changed)
     cache.connect2("cache_post_change", cache_post_changed)
-    print(("aptitude" in cache))
+    print("aptitude" in cache)
     pkg = cache["aptitude"]
     print(pkg.name)
     print(len(cache))
@@ -1028,7 +967,7 @@ def _test():
     pm = apt_pkg.PackageManager(cache._depcache)
     fetcher = apt_pkg.Acquire(apt.progress.text.AcquireProgress())
     cache._fetch_archives(fetcher, pm, None)
-    #sys.exit(1)
+    # sys.exit(1)
 
     print("Testing filtered cache (argument is old cache)")
     filtered = FilteredCache(cache)
@@ -1055,5 +994,5 @@ def _test():
     print(len(filtered))
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     _test()
